/*
 * SoftI2c.c
 *
 *  Created on: 2021316
 *      Author: Geralt
 */
#include "SoftI2c.h"

#define SetSCL()     (SOFTI2C_SCL_GPIO_PORT->PBSC = SOFTI2C_SCL_GPIO_PIN)
#define SetSDA()     (SOFTI2C_SDA_GPIO_PORT->PBSC = SOFTI2C_SDA_GPIO_PIN)
#define ResetSCL()   (SOFTI2C_SCL_GPIO_PORT->PBC  = SOFTI2C_SCL_GPIO_PIN)
#define ResetSDA()   (SOFTI2C_SDA_GPIO_PORT->PBC  = SOFTI2C_SDA_GPIO_PIN)
#define ReadSCL()    ((SOFTI2C_SCL_GPIO_PORT->PID) & SOFTI2C_SCL_GPIO_PIN)
#define ReadSDA()    ((SOFTI2C_SDA_GPIO_PORT->PID) & SOFTI2C_SDA_GPIO_PIN)

#if SOFTI2C_FAST_MODE
#define I2C_Delay()  {__NOP();__NOP();__NOP();}
#else
#define I2C_Delay()  SysTick_Delay_Us(5)
#endif

#define DelayMicroseconds(x) do{int loops = x; while(loops--) __NOP();}while(0)


void I2C_Init() {
	SetSCL();
	I2C_Delay();
	SetSDA();
	I2C_Delay();
}

/*
I2C_Start()
ܣI2Cߵʼ״̬
˵
 SCLڸߵƽڼ䣬SDA½ʱI2C
 SDASCLʲôƽ״̬ȷʼ״̬
 Ҳظʼ״̬
 ִкI2Cߴæ״̬
*/
void I2C_Start() {
	SetSCL();
	I2C_Delay();
	SetSDA();
	DelayMicroseconds(5);
	ResetSDA();  //ʼź
	I2C_Delay();
	ResetSCL();  //ǯסI2Cߣ׼ͻ
	DelayMicroseconds(20);
}

/*
I2C_Stop()
ܣI2Cߵֹͣ״̬
˵
 SCLڸߵƽڼ䣬SDAʱֹͣI2C
 SDASCLʲôƽ״̬ȷֹͣ״̬
 ִкI2Cߴڿ״̬
*/
void I2C_Stop() {

	ResetSDA();  //ͽź
	I2C_Delay();

	SetSCL();    //ͽʱź
	I2C_Delay();
	SetSDA();    //I2C߽ź
	I2C_Delay();
	DelayMicroseconds(20);
}

/*
 I2C_GetAck()
 ܣȡӻӦλ
 أ
 0ӻӦ
 1ӻӦ
 ˵
 ӻյÿֽڵݺҪӦλ
 ӻյ1ֽڵݺһҪӦλ
 */
uint8_t I2C_GetAck() {
	uint8_t ack;
	uint8_t Error_time = 255;

	I2C_Delay();
	SetSDA(); /*8λͷߣ׼Ӧλ ͷ*/
	I2C_Delay();
	SetSCL(); /**/
	I2C_Delay();
	do {
		ack = ReadSDA();
		Error_time--;
		if (Error_time == 0) {
			ResetSCL();
			I2C_Delay();
			return 1;
		}
	} while (ack);   //жǷյӦź

	ResetSCL();  //ʱߣǯסI2CԱ
	DelayMicroseconds(20);
	return 0;
}

/*
 I2C_PutAck()
 ܣӦλӦλ
 
 ack=0Ӧλ
 ack=1Ӧλ
 ˵
 ڽÿһֽڵݺ󣬶ӦӦλ
 ڽһֽڵݺӦӦλ
 */
void I2C_PutAck(uint8_t ack) {

	ack ? SetSDA() : ResetSDA();
	I2C_Delay();
	SetSCL();   //Ӧ
	I2C_Delay();

	ResetSCL();  //ʱߣǯסI2CԱ  ռ
	DelayMicroseconds(20);
}

/*
I2C_Write()
ܣI2Cд1ֽڵ

 datҪдϵ
*/
void I2C_Write(uint8_t dat) {
	for (uint8_t t = 0; t < 8; t++) {
		(dat & 0x80) ? SetSDA() : ResetSDA();
		I2C_Delay();
		SetSCL();  //ʱΪߣ֪ͨʼλ
		I2C_Delay();
		ResetSCL();
		I2C_Delay();
		dat <<= 1;
	}
}

/*
 I2C_Read()
 ܣӴӻȡ1ֽڵ
 أȡһֽ
 */
uint8_t I2C_Read() {
	uint8_t dat = 0;
	I2C_Delay();
	I2C_Delay();
	SetSDA(); //ڶȡ֮ǰҪSDA

	I2C_Delay();

	for (uint8_t t = 0; t < 8; t++) {
		ResetSCL();   //
		I2C_Delay();
		SetSCL();     // ʱΪʹЧ
		I2C_Delay();
		dat <<= 1;
		if(ReadSDA()) dat += 1;
	}
	ResetSCL();
	I2C_Delay();
	return dat;
}

SoftI2cStatus SoftI2c_Init(void);
SoftI2cStatus SoftI2c_WriteByte(uint8_t address, uint8_t data);
SoftI2cStatus SoftI2c_ReadByte(uint8_t address, uint8_t* data);

SoftI2cStatus SoftI2c_MemWrite(uint8_t address, uint8_t* reg, uint8_t regSize, uint8_t* data, uint16_t dataSize) {
	// ȷַһλ0
	address &= 0xFE;
	// ź
	I2C_Start();
	// ʹӻַ
	I2C_Write(address);
	// ȴACKӦ
	if (I2C_GetAck()) {
		I2C_Stop();
		return I2C_NO_ACK;
	}
	// ͼĴַ
	while(regSize--){
		I2C_Write(*reg++);
		if (I2C_GetAck()) {
			I2C_Stop();
			return I2C_NO_ACK;
		}
	}
	// 
	while(dataSize--){
		I2C_Write(*data++);
		if (I2C_GetAck()) {
			I2C_Stop();
			return I2C_NO_ACK;
		}
	}

	return I2C_OK;
}

SoftI2cStatus SoftI2c_MemRead(uint8_t address, uint8_t* reg, uint8_t regSize, uint8_t* data, uint16_t dataSize) {
	// ȷַһλ0
	address &= 0xFE;
	// ź
	I2C_Start();
	// ʹӻַ
	I2C_Write(address);
	// ȴACKӦ
	if (I2C_GetAck()) {
		I2C_Stop();
		return I2C_NO_ACK;
	}
	// ͼĴַ
	while (regSize--) {
		I2C_Write(*reg++);
		if (I2C_GetAck()) {
			I2C_Stop();
			return I2C_NO_ACK;
		}
	}
	// ظʼ
	address |= 0x01;
	I2C_Start();
	I2C_Write(address);
	// ȴACKӦ
	if (I2C_GetAck()) {
		I2C_Stop();
		return I2C_NO_ACK;
	}

	//
	while(1) {
		*data++ = I2C_Read();
		if (--dataSize == 0) {
			I2C_PutAck(1);
			break;
		}
		I2C_PutAck(0);
	}

	I2C_Stop();
	return I2C_OK;
}
